'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib = require('tslib');
var React = require('react');
var reactWindowGrid = require('react-window-grid');
var Draggable = _interopDefault(require('react-draggable'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function InputCheckbox(props) {
  var checkbox = React.useRef();
  React.useEffect(function () {
    if (checkbox && checkbox.current) {
      checkbox.current.indeterminate = props.indeterminate === 'true';
    }
  });
  return React.createElement("input", _extends({
    ref: checkbox,
    type: "checkbox"
  }, props));
}

var DEFAULT_COLUMN_WIDTH = 150;

var noop = function () {};

function getLabel(column) {
  if (column === void 0) {
    column = {};
  }

  var label = '';

  if (column.label) {
    label = column.label;
  } else {
    label = column.name.replace(/([a-z])([A-Z])/g, function (all, p1, p2) {
      return p1 + ' ' + p2.toLowerCase();
    }).replace(/\s+/g, ' ');
  }

  return label.trim();
}

function useColumns(_a) {
  var columns = _a.columns,
      columnWidth = _a.columnWidth,
      checkbox = _a.checkbox;
  var normalize = React.useMemo(function () {
    var _tmp = columns;

    if (typeof _tmp === 'object' && !Array.isArray(_tmp)) {
      _tmp = Object.keys(_tmp).map(function (e) {
        return tslib.__assign(tslib.__assign({}, _tmp[e]), {
          name: e
        });
      });
    }

    if (!!checkbox) {
      _tmp = tslib.__spreadArrays([{
        _system: true,
        name: '___CHECKBOX',
        textAlign: 'center',
        width: 40,
        // @ts-ignore
        header: function (arg1, arg2, _a) {
          var selectedStatus = _a.selectedStatus;
          var checked = selectedStatus === 'all';
          var indeterminate = selectedStatus === 'some' ? 'true' : 'false';
          return React.createElement(InputCheckbox, {
            type: "checkbox",
            "data-rwt-checkbox-control": true,
            checked: checked,
            indeterminate: indeterminate,
            onChange: noop
          });
        },
        render: function (arg1, arg2, arg3) {
          var rowIndex = arg3.rowIndex,
              columnIndex = arg3.columnIndex,
              _key = arg3._key,
              isSelected = arg3.isSelected,
              _isChildRow = arg3._isChildRow;
          var checked = !!isSelected;

          if (_isChildRow) {
            return null;
          }

          return React.createElement("input", {
            type: "checkbox",
            "data-rwt-checkbox": true,
            "data-row-key": _key,
            checked: checked,
            onChange: noop
          });
        }
      }], _tmp);
    }

    return (_tmp || []).filter(function (column) {
      return column && (typeof column === 'string' || typeof column === 'object');
    }).map(function (column) {
      return typeof column === 'string' ? {
        name: column
      } : tslib.__assign({}, column);
    }).filter(function (column) {
      return column.name;
    }).map(function (column) {
      // const label = getLabel(column);
      // console.log(label);
      return tslib.__assign(tslib.__assign({}, column), {
        label: getLabel(column),
        // render: () => '---',
        render: typeof column.render === 'function' ? column.render : function (data) {
          return data;
        }
      });
    });
  }, [columns, !!checkbox]);
  var getColumnWidth = React.useMemo(function () {
    var _columnWidth = typeof columnWidth === 'function' ? columnWidth : function () {
      return columnWidth || DEFAULT_COLUMN_WIDTH;
    };

    return function (columnIndex) {
      var column = normalize[columnIndex] || {};
      return column.width || _columnWidth(columnIndex, column);
    };
  }, [normalize, columnWidth || null]);
  var fixedLeftCount = React.useMemo(function () {
    var count = normalize.filter(function (e) {
      return e.fixed;
    }).length;

    if (count > 0 && !!checkbox) {
      count += 1;
    }

    return count;
  }, [normalize, !!checkbox]);
  return [normalize, getColumnWidth, fixedLeftCount];
}

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".useDragHandle-module_cursorResize__1hqrp {\n  cursor: ew-resize;\n}\n";
var styles = {"cursorResize":"useDragHandle-module_cursorResize__1hqrp"};
styleInject(css);

function useDragHandle(columns, columnWidth, fixedLeftCount, container, resizeHp, handleResizeEnd, sizeInfo) {
  var _a = React.useState('tmp'),
      resizableKey = _a[0],
      setResizableKey = _a[1];

  var handleResize = React.useRef();
  handleResize.current = handleResizeEnd;
  var aaa = React.useRef({
    baseWidth: 0,
    divEl: null
  });

  var _b = React.useMemo(function () {
    var handleDragStart = function (event, data) {
      var node = data.node;
      var columnKey = node.getAttribute('data-column-key');
      var index = columns.findIndex(function (e) {
        return e.name === columnKey;
      });
      node.style.visibility = 'hidden'; // const left = columns.slice(0, index).reduce((prev, curr, index) => prev + columnWidth(index), 0);
      //

      var width = columnWidth(index);
      var _width = width;
      var left = node.getBoundingClientRect().right - width - container.current.getBoundingClientRect().left;
      var fixedLeftWidth = columns.slice(0, fixedLeftCount).reduce(function (prev, curr, index) {
        return prev + columnWidth(index);
      }, 0);

      if (left < fixedLeftWidth && fixedLeftWidth < left + width) {
        width -= fixedLeftWidth - left;
        left = fixedLeftWidth;
      }

      var column = columns[index];
      aaa.current.baseWidth = width;
      aaa.current._baseWidth = _width;
      aaa.current.baseLeft = left;
      aaa.current.fixedLeftWidth = fixedLeftWidth; // @ts-ignore

      aaa.current.minWidth = column && column.minWidth || 80;
      aaa.current._minWidth = Math.max(0, aaa.current.minWidth - (_width - aaa.current.baseWidth)); // aaa.current._baseWidth = width;

      aaa.current.columnKey = columnKey;

      if (resizeHp.current) {
        var divEl = document.createElement('div');
        resizeHp.current.innerHTML = '';
        resizeHp.current.appendChild(divEl);
        divEl.style.left = left + "px";
        divEl.style.width = width + "px";

        if (sizeInfo.current) {
          divEl.style.height = sizeInfo.current.clientHeight + "px";
        }

        aaa.current.divEl = divEl;
        document.body.classList.add(styles.cursorResize);
      }
    };

    var handleDrag = function (event, data) {
      var width = Math.max(aaa.current._minWidth, aaa.current.baseWidth + data.x);

      if (sizeInfo.current) {
        var clintWidth = sizeInfo.current.clientWidth;

        if (width > clintWidth - aaa.current.baseLeft) {
          width = clintWidth - aaa.current.baseLeft;
        }

        aaa.current.divEl.style.width = width + "px";
      }
    };

    var resetResizable = function (event, data) {
      document.body.classList.remove(styles.cursorResize);
      var width = Math.max(aaa.current.minWidth, aaa.current._baseWidth + data.x); // data.node.style.visibility = 'visible';
      // data.node.style.transform = '';      // handleResize.current

      if (aaa.current && typeof handleResize.current === 'function') {
        var columnKey = aaa.current.columnKey;
        handleResize.current({
          name: columnKey,
          width: width
        });
      }

      if (resizeHp.current) {
        resizeHp.current.innerHTML = '';
      }

      setResizableKey(Math.random() + '_'); // return false;
    };

    return [handleDragStart, handleDrag, resetResizable];
  }, [columns, columnWidth]),
      onStart = _b[0],
      onDrag = _b[1],
      onStop = _b[2];

  return {
    key: resizableKey,
    onStart: onStart,
    onDrag: onDrag,
    onStop: onStop
  };
}

var serialize = function () {
  var replacer = function (key, value) {
    return typeof value === 'function' ? value.toString() : value;
  };

  return function (value) {
    return JSON.stringify(value, replacer);
  };
}();

function useEventHandlers(ownEvents, userEvents, rows, context) {
  var dict = {
    click: 'onClick',
    mouseover: 'onMouseOver',
    mouseout: 'onMouseOut'
  };
  var eventsHash = serialize(userEvents);
  var eventHandlers = React.useMemo(function () {
    var entries = function (target) {
      return Object.keys(target).map(function (k) {
        return [k, target[k]];
      });
    }; // props.events
    // console.log('%c-=-=-=-=-=-=-=-=-', 'background:orange');
    // console.log();
    // console.log();


    var getCellElement = function (source) {
      var SELECTOR = '[data-row-index][data-column-index][data-column]';
      return source.matches(SELECTOR) ? source : source.closest(SELECTOR);
    };

    var getEventTarget = function (source, selector) {
      var target = source.matches(selector) ? source : source.closest(selector);

      if (target) {
        var cellElement = getCellElement(target);

        if (cellElement) {
          var rowIndex_ = cellElement.getAttribute('data-row-index');
          var columnIndex_ = cellElement.getAttribute('data-column-index');
          var column = cellElement.getAttribute('data-column');

          if (rowIndex_ && columnIndex_) {
            var rowIndex = ~~rowIndex_;
            var columnIndex = ~~columnIndex_;
            var row = rows[rowIndex_] || {};
            var _key = row._key,
                _index = row._index,
                _isHeader = row._isHeader,
                _isChildRow = row._isChildRow,
                data = row.obj;
            return {
              target: target,
              rowIndex: rowIndex,
              columnIndex: columnIndex,
              column: column,
              data: data,
              context: context,
              // @ts-ignore
              _key: _key,
              _index: _index,
              _isHeader: !!_isHeader,
              _isChildRow: !!_isChildRow
            };
          }
        }
      }

      return null;
    };

    var handlerFactory = function (details) {
      var handles = entries(details).map(function (_a) {
        var selector = _a[0],
            handler = _a[1];
        return {
          selector: selector,
          handler: handler
        };
      });
      return function (event) {
        // console.log(event.target);
        // console.log(handles);
        handles.forEach(function (e) {
          var target = getEventTarget(event.target, e.selector);

          if (target) {
            e.handler(event, target);
          } // console.log(target);

        });
      };
    };

    var mergedEvents = tslib.__spreadArrays(Object.entries(ownEvents || {}), Object.entries(userEvents || {})).reduce(function (accum, _a) {
      var _b;

      var k = _a[0],
          v = _a[1];
      return tslib.__assign(tslib.__assign({}, accum), (_b = {}, _b[k] = tslib.__assign(tslib.__assign({}, accum[k]), v), _b));
    }, {});

    return entries(mergedEvents).reduce(function (accum, _a) {
      var _b;

      var eventName = _a[0],
          details = _a[1];
      var handlerName = dict[eventName.toLowerCase()];

      if (handlerName) {
        return tslib.__assign(tslib.__assign({}, accum), (_b = {}, _b[handlerName] = handlerFactory(details), _b));
      }

      return accum;
    }, {});
  }, [ownEvents, eventsHash, rows, context]);
  return eventHandlers;
}

function useHandle(callback) {
  var handle = React.useRef();
  handle.current = callback;
  return function () {
    var args = [];

    for (var _i = 0; _i < arguments.length; _i++) {
      args[_i] = arguments[_i];
    }

    if (typeof handle.current === 'function') {
      handle.current.apply(handle, args);
    }
  };
}

var DEFAULT_ROW_HEIGHT = 40;

var isDefined = function (e) {
  return typeof e !== 'undefined';
};

function useGetChildRows(childRows) {
  return React.useMemo(function () {
    return typeof childRows === 'function' ? childRows : function () {
      return [];
    };
  }, [childRows]);
}

function useContext(context) {
  return React.useMemo(function () {
    return context || {};
  }, [context || null]);
}

function useRows(_a) {
  var _this = this;

  var rows = _a.rows,
      rowHeight = _a.rowHeight,
      columns = _a.columns,
      checkbox = _a.checkbox,
      getChildRows = _a.getChildRows,
      trackBy = _a.trackBy,
      context = _a.context;

  var _getChildRows = useGetChildRows(getChildRows);

  var _context = useContext(context);

  var toRowObj = React.useMemo(function () {
    var availColumns = !!checkbox ? columns.slice(1) : columns;
    return function (row) {
      return Array.isArray(row) ? availColumns.reduce(function (accum, _a, i) {
        var _b;

        var name = _a.name;
        return tslib.__assign(tslib.__assign({}, accum), (_b = {}, _b[name] = row[i], _b));
      }, {}) : row;
    };
  }, [columns, !!checkbox]);
  var toRowValues = React.useMemo(function () {
    return function (rowObj) {
      return columns.map(function (_a) {
        var name = _a.name,
            getValue = _a.getValue;
        var value = rowObj[name];
        return typeof getValue === 'function' ? getValue(value, rowObj, _context) : value;
      });
    };
  }, [columns, _context]);
  var normalize = React.useMemo(function () {
    return function (row, _index, _childIndex) {
      var obj = toRowObj(row);

      var _key = (typeof trackBy === 'function' ? trackBy(obj) : [_index, _childIndex].filter(isDefined).join('_')).toString();

      return {
        obj: obj,
        arr: toRowValues(obj),
        _key: _key,
        _index: _index,
        _childIndex: _childIndex,
        _isChildRow: typeof _childIndex !== 'undefined'
      };
    };
  }, [toRowObj, toRowValues, trackBy || null]);
  var getRows = React.useMemo(function () {
    return function (rows) {
      return tslib.__spreadArrays([// header
      {
        obj: {},
        arr: columns.map(function (_a) {
          var name = _a.name;
          return name;
        }),
        _isHeader: true
      }], rows.reduce(function (accum, row, i) {
        var rowObj = toRowObj(row);

        var data = tslib.__spreadArrays([// parent row
        normalize(rowObj, i)], _getChildRows(rowObj).map(function (childRow, j) {
          return normalize(childRow, i, j);
        }));

        return tslib.__spreadArrays(accum, data);
      }, []));
    };
  }, [columns, toRowObj, normalize, _getChildRows]);
  var normalized = React.useMemo(function () {
    return getRows(rows);
  }, [getRows, rows]);
  var dataRows = React.useMemo(function () {
    var baseIndex = normalized.findIndex(function (e) {
      return e._isHeader !== true;
    });
    return normalized.slice(baseIndex);
  }, [normalized]);
  var getRowHeight = React.useMemo(function () {
    var _rowHeight = typeof rowHeight === 'function' ? rowHeight : function () {
      return rowHeight || DEFAULT_ROW_HEIGHT;
    };

    return function (rowIndex) {
      return _rowHeight(rowIndex, normalized[rowIndex]);
    };
  }, [normalized, rowHeight || null]);
  var getData = React.useMemo(function () {
    return function (data) {
      return tslib.__awaiter(_this, void 0, void 0, function () {
        var columns_, rows_, startIndex;

        var _this = this;

        return tslib.__generator(this, function (_a) {
          switch (_a.label) {
            case 0:
              columns_ = (columns || []).map(function (column) {
                return tslib.__assign({}, column);
              });
              rows_ = (Array.isArray(data) ? getRows(data) : rows) || [];
              startIndex = columns_.findIndex(function (column) {
                return !column._system;
              });
              rows_ = rows_.filter(function (row) {
                return !row._isHeader;
              });
              return [4
              /*yield*/
              , columns_.reduce(function (prevRows, col, i) {
                return tslib.__awaiter(_this, void 0, void 0, function () {
                  var prev, nextRows;

                  var _this = this;

                  return tslib.__generator(this, function (_a) {
                    switch (_a.label) {
                      case 0:
                        if (!(typeof col.batchData === 'function')) return [3
                        /*break*/
                        , 3];
                        return [4
                        /*yield*/
                        , prevRows];

                      case 1:
                        prev = _a.sent();
                        return [4
                        /*yield*/
                        , Promise.all(prev.map(function (row, j) {
                          return tslib.__awaiter(_this, void 0, void 0, function () {
                            var newRow, _a, _b;

                            return tslib.__generator(this, function (_c) {
                              switch (_c.label) {
                                case 0:
                                  newRow = tslib.__spreadArrays(row.arr || []);
                                  _a = newRow;
                                  _b = i;
                                  return [4
                                  /*yield*/
                                  , col.batchData(newRow[i], tslib.__assign({}, row.obj), context)];

                                case 1:
                                  _a[_b] = _c.sent();
                                  return [2
                                  /*return*/
                                  , tslib.__assign(tslib.__assign({}, row), {
                                    arr: newRow
                                  })];
                              }
                            });
                          });
                        }))];

                      case 2:
                        nextRows = _a.sent();
                        return [2
                        /*return*/
                        , nextRows];

                      case 3:
                        return [2
                        /*return*/
                        , prevRows];
                    }
                  });
                });
              }, Promise.resolve(rows_))];

            case 1:
              rows_ = _a.sent();
              return [2
              /*return*/
              , {
                columns: columns_.slice(startIndex),
                rows: rows_.map(function (row) {
                  return row.arr.slice(startIndex);
                })
              }];
          }
        });
      });
    };
  }, [columns, getRows, rows, context]);
  return [normalized, dataRows, getRowHeight, getRows, getData];
}

var theme = function (classNames) {
  var _a, _b, _c, _d;

  var borderColor1 = '#d3d3d3';
  var borderColor2 = '#dadcde';
  return _a = {
    borderTop: "1px solid " + borderColor1,
    borderBottom: "1px solid " + borderColor1,
    boxSizing: 'content-box'
  }, _a[classNames.CELL] = {
    display: 'flex',
    fontSize: 12.5,
    padding: '0 1rem',
    // borderRight: `1px solid ${borderColor2}`,
    borderBottom: "1px solid " + borderColor2,
    alignItems: 'center'
  }, _a[classNames.HEADER] = {
    background: '#f5f6f7',
    borderBottom: "1px solid " + borderColor1,
    fontWeight: 700
  }, _a[classNames.IS_SCROLLING] = (_b = {}, _b[classNames.CELL] = {
    pointerEvents: 'none'
  }, _b), _a[classNames.SECTION_CENTER_V] = (_c = {}, _c[classNames.ROW_LAST] = {
    borderBottomColor: 'transparent'
  }, _c), _a[classNames.SECTION_BOTTOM] = (_d = {}, _d[classNames.CELL] = {
    borderTop: "1px solid " + borderColor2,
    borderBottom: 'none'
  }, _d), _a;
};

var theme$1 = function (classNames) {
  var _a, _b, _c, _d;

  var borderColor1 = '#d3d3d3';
  var borderColor2 = '#dadcde';
  return _a = {
    borderTop: "1px solid " + borderColor1,
    borderBottom: "1px solid " + borderColor1,
    boxSizing: 'content-box'
  }, _a[classNames.CELL] = {
    display: 'flex',
    fontSize: 12.5,
    padding: '0 1rem',
    // borderRight: `1px solid ${borderColor2}`,
    borderBottom: "1px solid " + borderColor2,
    alignItems: 'center'
  }, _a[classNames.HEADER] = {
    // background: '#f5f6f7',
    background: 'yellow',
    borderBottom: "1px solid " + borderColor1,
    fontWeight: 700
  }, _a[classNames.IS_SCROLLING] = (_b = {}, _b[classNames.CELL] = {
    pointerEvents: 'none'
  }, _b), _a[classNames.SECTION_CENTER_V] = (_c = {}, _c[classNames.ROW_LAST] = {
    borderBottomColor: 'transparent'
  }, _c), _a[classNames.SECTION_BOTTOM] = (_d = {}, _d[classNames.CELL] = {
    borderTop: "1px solid " + borderColor2,
    borderBottom: 'none'
  }, _d), _a;
};

var theme$2 = function (classNames) {
  var _a, _b, _c, _d;

  var borderColor1 = '#d3d3d3';
  var borderColor2 = '#dadcde';
  return _a = {
    borderTop: "1px solid " + borderColor1,
    borderBottom: "1px solid " + borderColor1,
    boxSizing: 'content-box'
  }, _a[classNames.CELL] = {
    display: 'flex',
    fontSize: 12.5,
    padding: '0 1rem',
    // borderRight: `1px solid ${borderColor2}`,
    borderBottom: "1px solid " + borderColor2,
    alignItems: 'center'
  }, _a[classNames.HEADER] = {
    // background: '#f5f6f7',
    background: 'orange',
    borderBottom: "1px solid " + borderColor1,
    fontWeight: 700
  }, _a[classNames.IS_SCROLLING] = (_b = {}, _b[classNames.CELL] = {
    pointerEvents: 'none'
  }, _b), _a[classNames.SECTION_CENTER_V] = (_c = {}, _c[classNames.ROW_LAST] = {
    borderBottomColor: 'transparent'
  }, _c), _a[classNames.SECTION_BOTTOM] = (_d = {}, _d[classNames.CELL] = {
    borderTop: "1px solid " + borderColor2,
    borderBottom: 'none'
  }, _d), _a;
};

var themes = {
  basic: theme,
  bordered: theme$1,
  striped: theme$2
};

function useTheme(theme) {
  return React.useMemo(function () {
    if (typeof theme === 'function') {
      return theme;
    } // if (typeof theme === 'string' && themes[theme]) {
    // }
    // @ts-ignore


    return themes[theme] || themes.basic; // if (typeof theme === 'string' && themes[theme]) {
    //   return themes[theme];
    // }
  }, [theme || null]); // return theme;
}

var css$1 = ".WindowTable-module_root__10H_u {\n  display: block;\n  position: relative;\n}\n\n.WindowTable-module_root__10H_u .WindowTable-module_cell__3H7bY {\n  display: flex;\n  align-items: center;\n  justify-content: left;\n  padding: 0 .5rem;\n}\n\n.WindowTable-module_root__10H_u .WindowTable-module_cell__3H7bY.WindowTable-module_text-right__1owe3 {\n  text-align: right;\n  justify-content: flex-end;\n}\n\n.WindowTable-module_root__10H_u .WindowTable-module_cell__3H7bY.WindowTable-module_text-left__2-jqB {\n  text-align: left;\n  justify-content: flex-start;\n}\n\n.WindowTable-module_root__10H_u .WindowTable-module_cell__3H7bY.WindowTable-module_text-center__2hLEV {\n  text-align: center;\n  justify-content: center;\n}\n\n.WindowTable-module_ellipsis__-cTbl {\n  text-overflow: ellipsis;\n  white-space: nowrap;\n  overflow: hidden;\n}\n\n.WindowTable-module_resizeHelper__CnENF > div {\n  position: absolute;\n  top: 0;\n  border-left: 1px solid rgba(0, 0, 0, .15);\n  border-right: 1px solid rgba(0, 0, 0, .15);\n  border-left: 1px solid #1e88e5;\n  border-right: 1px solid #1e88e5;\n  border-left: 1px solid #1e88e5;\n  border-right: 1px solid #1e88e5;\n  background: rgba(0, 0, 0, .1);\n  background: rgba(30, 136, 229, .1);\n}\n\n.WindowTable-module_resizeHandle__E7cLL {\n  position: absolute;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  width: 6px;\n  background: rgba(0, 0, 0, 0);\n  cursor: ew-resize;\n}\n\n.WindowTable-module_resizeHandle__E7cLL:hover {\n  background: #1e88e5;\n  background: #1e88e5;\n}\n\n.WindowTable-module_cursorResize__2jBjr {\n  cursor: ew-resize;\n}\n\n.wow {\n  border: 2px solid blue;\n}\n";
var styles$1 = {"root":"WindowTable-module_root__10H_u","cell":"WindowTable-module_cell__3H7bY","text-right":"WindowTable-module_text-right__1owe3","text-left":"WindowTable-module_text-left__2-jqB","text-center":"WindowTable-module_text-center__2hLEV","ellipsis":"WindowTable-module_ellipsis__-cTbl","resizeHelper":"WindowTable-module_resizeHelper__CnENF","resizeHandle":"WindowTable-module_resizeHandle__E7cLL","cursorResize":"WindowTable-module_cursorResize__2jBjr"};
styleInject(css$1);

var cancelMouseDown = function (e) {
  e.preventDefault();
};

var toValidContent = function (e) {
  var type = typeof e;

  if (type === 'string' || type === 'number' || React.isValidElement(e)) {
    return e;
  }

  if (e && typeof e.toString === 'function') {
    return e.toString();
  }

  return null;
};

var WindowTable = function (props, ref) {
  var context = React.useMemo(function () {
    return props.context || {};
  }, [props.context || null]);

  var _a = useColumns({
    columns: props.columns,
    columnWidth: props.columnWidth,
    checkbox: props.checkbox
  }),
      columns = _a[0],
      columnWidth = _a[1],
      fixedLeftCount = _a[2];

  var _b = useRows({
    rows: props.rows,
    rowHeight: props.rowHeight,
    columns: columns,
    context: context,
    checkbox: props.checkbox,
    getChildRows: props.getChildRows,
    trackBy: props.trackBy
  }),
      rows = _b[0],
      dataRows = _b[1],
      rowHeight = _b[2],
      getRows = _b[3];

  var maxHeight = React.useMemo(function () {
    return props.maxHeight || 400;
  }, [props.maxHeight || null]);
  var fixedTopCount = (props.fixedTopCount || 0) + 1;

  var _c = React.useState({
    rowIndex: null,
    columnIndex: null
  }),
      hover = _c[0],
      setHover = _c[1];

  var currHover = React.useRef(hover);
  currHover.current = hover;
  var handleRef = React.useRef(setHover);
  handleRef.current = setHover;
  var timer = React.useRef();
  var styleRef = React.useRef(); // const
  // const
  // @ts-ignore

  var _d = React.useState(props.selected || []),
      selected = _d[0],
      setSelected = _d[1]; // cibst


  var selectedRef = React.useRef([]);
  selectedRef.current = selected; // @ts-ignore
  // @ts-ignore

  var classNames = React.useMemo(function () {
    return tslib.__assign({
      HEADER: 'header',
      CELL: 'cell'
    }, props.classNames || {});
  }, [// @ts-ignore
  props.classNames || null]);
  var ownEvents = React.useMemo(function () {
    var _a;

    return {
      // mouseover: {
      //   '.cell[data-row-index][data-column-index]': (event: SyntheticEvent, ui: any) => {
      //     timer.current && clearTimeout(timer.current);
      //     styleRef.current.innerHTML = '';
      //     var styleNode = document.createElement('style');
      //     styleNode.type = 'text/css';
      //     var styleText = document.createTextNode(
      //       `[data-row-index="${ui.rowIndex}"] { background:#e9eaeb !important } `,
      //     );
      //     styleNode.appendChild(styleText);
      //     styleRef.current.appendChild(styleNode);
      //   },
      // },
      // mouseout: {
      //   '.cell[data-row-index]': (event: SyntheticEvent, ui: any) => {
      //     styleRef.current.innerHTML = '';
      //   },
      // },
      click: (_a = {}, _a["." + classNames.CELL + "[data-row-index] input[type=checkbox][data-rwt-checkbox-control]"] = function (event, ui) {
        var parentRows = dataRows.filter(function (e) {
          return !e._isChildRow;
        }); // console.log('>%-=-=-=-=-=-=-', 'background:yellow')

        if (selectedRef.current.length < parentRows.length) {
          setSelected(parentRows.map(function (e) {
            return e._key;
          }).sort());
        } else {
          setSelected([]);
        }
      }, _a["." + classNames.CELL + "[data-row-index] input[type=checkbox][data-rwt-checkbox]"] = function (event, ui) {
        var _selected = selectedRef.current.reduce(function (accum, curr) {
          var _a;

          return tslib.__assign(tslib.__assign({}, accum), (_a = {}, _a[curr] = true, _a));
        }, {});

        if (_selected[ui._key]) {
          delete _selected[ui._key];
        } else {
          _selected[ui._key] = true;
        }

        setSelected(Object.keys(_selected).sort());
      }, _a)
    };
  }, [classNames, dataRows]); // @ts-ignore

  var eventHandlers = useEventHandlers(ownEvents, props.events, rows, context); // const [resizeHelper, setResizeHelper] = useState();

  var resizeHelper = React.useRef();
  var container = React.useRef(); // const [sizeInfo, setSizeInfo] = useState({});

  var sizeInfoRef = React.useRef({});

  var handleResize = function (info) {
    sizeInfoRef.current = info; // setSizeInfo(info);
  };

  var _e = useDragHandle(columns, columnWidth, fixedLeftCount, container, resizeHelper, props.onColumnResizeEnd, sizeInfoRef),
      resizableKey = _e.key,
      handleDragStart = _e.onStart,
      handleDrag = _e.onDrag,
      handleDragStop = _e.onStop;

  var selectedStatus = React.useMemo(function () {
    if (selected.length === 0) {
      return 'none';
    }

    return dataRows.filter(function (_a) {
      var _key = _a._key,
          _isChildRow = _a._isChildRow;
      return !_isChildRow && _key;
    }).map(function (_a) {
      var _key = _a._key;
      return _key;
    }).sort().join(',') === selected.sort().join(',') ? 'all' : 'some';
  }, [dataRows, selected]);
  var renderHeader = React.useMemo(function () {
    return function (data, column) {
      var txt = column.label;

      if (typeof props.renderHeader === 'function') {
        txt = props.renderHeader(data, column);
      }

      if (typeof props.onColumnResizeEnd !== 'function') {
        return txt;
      }

      return React.createElement(React.Fragment, null, txt, React.createElement("div", null, React.createElement(Draggable, {
        key: resizableKey,
        onStart: handleDragStart,
        onDrag: handleDrag,
        onStop: handleDragStop,
        axis: "x"
      }, React.createElement("div", {
        className: styles$1.resizeHandle,
        "data-column-key": column.name
      }))));
    };
  }, [props.renderHeader, resizableKey, handleDragStart, handleDragStop, props.onColumnResizeEnd || null]);
  var renderCore = React.useMemo(function () {
    var seletedMap = selected.reduce(function (accum, curr) {
      var _a;

      return tslib.__assign(tslib.__assign({}, accum), (_a = {}, _a[curr] = true, _a));
    }, {});

    var getIsSelected = function (row) {
      return !!seletedMap[row._key];
    };

    return function (rowIndex, columnIndex, style) {
      // @ts-ignore
      var row = rows[rowIndex];
      var column = columns[columnIndex];
      var data = row.arr[columnIndex];
      var isSelected = getIsSelected(row); // console.log(row);

      if (!column) {
        return null;
      } // let className = styles.cell;
      // // if (column.ellipsis !== false) {
      // //   className += ' ' + styles.ellipsis;
      // // }
      // // if (column.textAlign && styles['text-' + column.textAlign]) {
      // //   className += ' ' + styles['text-' + column.textAlign];
      // // }


      var rendered = null; // @ts-ignore

      if (row._isHeader) {
        if (column.header) {
          rendered = column.header(data, column, {
            selectedStatus: selectedStatus
          });
        } else {
          // @ts-ignore
          rendered = renderHeader(data, column, {
            selectedStatus: selectedStatus
          });
        }
      } else {
        var _key = row._key,
            _index = row._index; // @ts-ignore

        if (typeof props.renderCell === 'function') {
          // @ts-ignore
          rendered = props.renderCell(data, row.obj, column, tslib.__assign(tslib.__assign({}, context), {
            rowIndex: rowIndex,
            columnIndex: columnIndex,
            _key: _key,
            _index: _index,
            isSelected: isSelected,
            selectedStatus: selectedStatus,
            _isChildRow: !!row._isChildRow,
            style: style
          }));
        }

        if (rendered === null) {
          // @ts-ignore
          rendered = column.render(data, row.obj, tslib.__assign(tslib.__assign({}, context), {
            rowIndex: rowIndex,
            columnIndex: columnIndex,
            _key: _key,
            _index: _index,
            isSelected: isSelected,
            selectedStatus: selectedStatus,
            _isChildRow: !!row._isChildRow,
            style: style
          }));
        }
      }

      rendered = toValidContent(rendered);
      return rendered; // return (
      //   <div className={className} data-column={column.name}>
      //     {rendered}
      //   </div>
      // );
    };
  }, [rows, columns, renderHeader, selected, selectedStatus]);
  var getClassNames = React.useMemo(function () {
    // return ()
    if (typeof props.getClassNames === 'function') {
      return function (rowIndex, columnIndex) {
        // console.log(rows[rowIndex])
        // @ts-ignore
        var row = rows[rowIndex];
        var tmp = {
          _rowIndex: row._index,
          _isChildRow: row._isChildRow
        };
        var column = columns[columnIndex]; // @ts-ignore

        return props.getClassNames(tslib.__assign({
          rowIndex: rowIndex,
          columnIndex: columnIndex
        }, tmp));
      };
    }

    return function () {
      return '';
    }; // @ts-ignore
  }, [props.classNames, rows, columns]);
  var Cell = React.useMemo(function () {
    // @ts-ignore
    return function (_a) {
      var rowIndex = _a.rowIndex,
          columnIndex = _a.columnIndex,
          className_ = _a.className,
          style = _a.style;
      var _style = style; // @ts-ignore

      if (columnIndex === -1) {
        _style = tslib.__assign(tslib.__assign({}, _style), {
          overflow: 'hidden',
          padding: 0
        });
      }

      var row = rows[rowIndex];
      var className__ = ' ' + styles$1.cell;
      var column = columns[columnIndex] || {};

      if (column.ellipsis !== false) {
        className__ += ' ' + styles$1.ellipsis;
      }

      if (column.textAlign && styles$1['text-' + column.textAlign]) {
        className__ += ' ' + styles$1['text-' + column.textAlign];
      }

      if (row._isHeader) {
        className__ += ' ' + classNames.HEADER;
      } // console.log()


      var className = className_ + className__ + (" " + getClassNames(rowIndex, columnIndex)); // dummy 컬럼은 없을 수 있다.

      var column_ = columns[columnIndex] || {};
      return React.createElement("div", {
        className: className,
        style: _style,
        "data-row-index": rowIndex,
        "data-column-index": columnIndex,
        "data-column": column_.name
      }, renderCore(rowIndex, columnIndex, _style));
    };
  }, [rows, columns, renderCore, getClassNames, classNames]);
  React.useEffect(function () {
    var keys = dataRows.filter(function (e) {
      return !e._isChildRow;
    }).reduce(function (accum, _a) {
      var _b;

      var _key = _a._key;
      return tslib.__assign(tslib.__assign({}, accum), (_b = {}, _b[_key] = true, _b));
    }, {});
    var availKeys = selected.filter(function (e) {
      return keys[e];
    });

    if (props.status !== 'PENDING' && selected.join(',') !== availKeys.join(',')) {
      setSelected(availKeys);
    }
  }, [props.status || null, dataRows]);
  var selectedProps = React.useRef('');
  React.useEffect(function () {
    if (Array.isArray(props.selected)) {
      var hash = props.selected.sort().join(',');

      if (Array.isArray(props.selected) && selectedProps.current !== hash && hash !== selected.sort().join(',')) {
        setSelected(props.selected);
      }

      selectedProps.current = hash;
    }
  }, [props.selected || null]);
  React.useEffect(function () {
    if (props.status !== 'PENDING' && typeof props.onSelect === 'function') {
      props.onSelect(selected);
    }
  }, [props.status || null, selected]); // console.log(selected);

  var theme = useTheme(props.theme);
  var guideline = React.useMemo(function () {
    return typeof props.guideline === 'undefined' ? true : !!props.guideline;
  }, [props.guideline || null]);
  var windowGrid = React.useRef();
  React.useImperativeHandle(ref, function () {
    return {
      getData: function (data) {
        return tslib.__awaiter(void 0, void 0, void 0, function () {
          var columns_, rows_, startIndex;
          return tslib.__generator(this, function (_a) {
            switch (_a.label) {
              case 0:
                columns_ = (columns || []).map(function (column) {
                  return tslib.__assign({}, column);
                });
                rows_ = (Array.isArray(data) ? getRows(data) : rows) || [];
                startIndex = columns_.findIndex(function (column) {
                  return !column._system;
                });
                rows_ = rows_.filter(function (row) {
                  return !row._isHeader;
                });
                return [4
                /*yield*/
                , columns_.reduce(function (prevRows, col, i) {
                  return tslib.__awaiter(void 0, void 0, void 0, function () {
                    var prev, nextRows;
                    return tslib.__generator(this, function (_a) {
                      switch (_a.label) {
                        case 0:
                          if (!(typeof col.batchData === 'function')) return [3
                          /*break*/
                          , 3];
                          return [4
                          /*yield*/
                          , prevRows];

                        case 1:
                          prev = _a.sent();
                          return [4
                          /*yield*/
                          , Promise.all(prev.map(function (row, j) {
                            return tslib.__awaiter(void 0, void 0, void 0, function () {
                              var newRow, _a, _b;

                              return tslib.__generator(this, function (_c) {
                                switch (_c.label) {
                                  case 0:
                                    newRow = tslib.__spreadArrays(row.arr || []);
                                    _a = newRow;
                                    _b = i;
                                    return [4
                                    /*yield*/
                                    , col.batchData(newRow[i], tslib.__assign({}, row.obj), context)];

                                  case 1:
                                    _a[_b] = _c.sent();
                                    return [2
                                    /*return*/
                                    , tslib.__assign(tslib.__assign({}, row), {
                                      arr: newRow
                                    })];
                                }
                              });
                            });
                          }))];

                        case 2:
                          nextRows = _a.sent();
                          return [2
                          /*return*/
                          , nextRows];

                        case 3:
                          return [2
                          /*return*/
                          , prevRows];
                      }
                    });
                  });
                }, Promise.resolve(rows_))];

              case 1:
                rows_ = _a.sent();
                return [2
                /*return*/
                , {
                  columns: columns_.slice(startIndex),
                  rows: rows_.map(function (row) {
                    return row.arr.slice(startIndex);
                  })
                }];
            }
          });
        });
      },
      scrollTo: function (params) {
        if (windowGrid.current && typeof windowGrid.current.scrollTo === 'function') {
          windowGrid.current.scrollTo(params);
        }
      }
    };
  }, [rows, columns, getRows, context]);
  var onVisibleRangeChange = useHandle(props.onVisibleRangeChange); // @ts-ignore

  return React.createElement("div", {
    ref: container,
    className: styles$1.root
  }, React.createElement("div", {
    ref: styleRef
  }), React.createElement("div", {
    onMouseMove: cancelMouseDown
  }, React.createElement("div", eventHandlers, React.createElement(reactWindowGrid.WindowGrid, _extends({
    ref: windowGrid
  }, props, {
    classNames: classNames,
    rowHeight: rowHeight,
    rowCount: rows.length,
    fixedLeftCount: fixedLeftCount,
    fixedTopCount: fixedTopCount,
    columnCount: columns.length,
    columnWidth: columnWidth,
    overscanCount: 2,
    overflow: rows.length > 1,
    fillerColumn: "append",
    onResize: handleResize,
    theme: theme,
    guideline: guideline,
    maxHeight: maxHeight,
    onVisibleRangeChange: onVisibleRangeChange
  }), Cell), React.createElement("div", {
    className: styles$1.resizeHelper,
    ref: resizeHelper
  }))));
};

var WindowTable$1 = React.forwardRef(WindowTable);

exports.WindowTable = WindowTable$1;
exports.useColumns = useColumns;
exports.useRows = useRows;
//# sourceMappingURL=react-window-table.cjs.js.map
